Struct HashMap

struct HashMap<K, V, S = crate::DefaultHashBuilder, A: Allocator = self::inner::Global> { ... }

A hash map implemented with quadratic probing and SIMD lookup.

The default hashing algorithm is currently foldhash, though this is subject to change at any point in the future. This hash function is very fast for all types of keys, but this algorithm will typically not protect against attacks such as HashDoS.

The hashing algorithm can be replaced on a per-HashMap basis using the default, with_hasher, and with_capacity_and_hasher methods. Many alternative algorithms are available on crates.io, such as the fnv crate.

It is required that the keys implement the Eq and Hash traits, although this can frequently be achieved by using #[derive(PartialEq, Eq, Hash)]. If you implement these yourself, it is important that the following property holds:

k1 == k2 -> hash(k1) == hash(k2)

In other words, if two keys are equal, their hashes must be equal.

It is a logic error for a key to be modified in such a way that the key's hash, as determined by the Hash trait, or its equality, as determined by the Eq trait, changes while it is in the map. This is normally only possible through Cell, RefCell, global state, I/O, or unsafe code.

It is also a logic error for the Hash implementation of a key to panic. This is generally only possible if the trait is implemented manually. If a panic does occur then the contents of the HashMap may become corrupted and some items may be dropped from the table.

Examples

use hashbrown::HashMap;

// Type inference lets us omit an explicit type signature (which
// would be `HashMap<String, String>` in this example).
let mut book_reviews = HashMap::new();

// Review some books.
book_reviews.insert(
    "Adventures of Huckleberry Finn".to_string(),
    "My favorite book.".to_string(),
);
book_reviews.insert(
    "Grimms' Fairy Tales".to_string(),
    "Masterpiece.".to_string(),
);
book_reviews.insert(
    "Pride and Prejudice".to_string(),
    "Very enjoyable.".to_string(),
);
book_reviews.insert(
    "The Adventures of Sherlock Holmes".to_string(),
    "Eye lyked it alot.".to_string(),
);

// Check for a specific one.
// When collections store owned values (String), they can still be
// queried using references (&str).
if !book_reviews.contains_key("Les Misérables") {
    println!("We've got {} reviews, but Les Misérables ain't one.",
             book_reviews.len());
}

// oops, this review has a lot of spelling mistakes, let's delete it.
book_reviews.remove("The Adventures of Sherlock Holmes");

// Look up the values associated with some keys.
let to_find = ["Pride and Prejudice", "Alice's Adventure in Wonderland"];
for &book in &to_find {
    match book_reviews.get(book) {
        Some(review) => println!("{}: {}", book, review),
        None => println!("{} is unreviewed.", book)
    }
}

// Look up the value for a key (will panic if the key is not found).
println!("Review for Jane: {}", book_reviews["Pride and Prejudice"]);

// Iterate over everything.
for (book, review) in &book_reviews {
    println!("{}: \"{}\"", book, review);
}

HashMap also implements an Entry API, which allows for more complex methods of getting, setting, updating and removing keys and their values:

use hashbrown::HashMap;

// type inference lets us omit an explicit type signature (which
// would be `HashMap<&str, u8>` in this example).
let mut player_stats = HashMap::new();

fn random_stat_buff() -> u8 {
    // could actually return some random value here - let's just return
    // some fixed value for now
    42
}

// insert a key only if it doesn't already exist
player_stats.entry("health").or_insert(100);

// insert a key using a function that provides a new value only if it
// doesn't already exist
player_stats.entry("defence").or_insert_with(random_stat_buff);

// update a key, guarding against the key possibly not being set
let stat = player_stats.entry("attack").or_insert(100);
*stat += random_stat_buff();

The easiest way to use HashMap with a custom key type is to derive Eq and Hash. We must also derive PartialEq.

use hashbrown::HashMap;

#[derive(Hash, Eq, PartialEq, Debug)]
struct Viking {
    name: String,
    country: String,
}

impl Viking {
    /// Creates a new Viking.
    fn new(name: &str, country: &str) -> Viking {
        Viking { name: name.to_string(), country: country.to_string() }
    }
}

// Use a HashMap to store the vikings' health points.
let mut vikings = HashMap::new();

vikings.insert(Viking::new("Einar", "Norway"), 25);
vikings.insert(Viking::new("Olaf", "Denmark"), 24);
vikings.insert(Viking::new("Harald", "Iceland"), 12);

// Use derived implementation to print the status of the vikings.
for (viking, health) in &vikings {
    println!("{:?} has {} hp", viking, health);
}

A HashMap with fixed list of elements can be initialized from an array:

use hashbrown::HashMap;

let timber_resources: HashMap<&str, i32> = [("Norway", 100), ("Denmark", 50), ("Iceland", 10)]
    .into_iter().collect();
// use the values stored in map

Implementations

impl<K, V, S> HashMap<K, V, S>

const fn with_hasher(hash_builder: S) -> Self

Creates an empty HashMap which will use the given hash builder to hash keys.

The hash map is initially created with a capacity of 0, so it will not allocate until it is first inserted into.

HashDoS resistance

The hash_builder normally use a fixed key by default and that does not allow the HashMap to be protected against attacks such as HashDoS. Users who require HashDoS resistance should explicitly use std::collections::hash_map::RandomState as the hasher when creating a HashMap.

The hash_builder passed should implement the BuildHasher trait for the HashMap to be useful, see its documentation for details.

Examples

use hashbrown::HashMap;
use hashbrown::DefaultHashBuilder;

let s = DefaultHashBuilder::default();
let mut map = HashMap::with_hasher(s);
assert_eq!(map.len(), 0);
assert_eq!(map.capacity(), 0);

map.insert(1, 2);
fn with_capacity_and_hasher(capacity: usize, hash_builder: S) -> Self

Creates an empty HashMap with the specified capacity, using hash_builder to hash the keys.

The hash map will be able to hold at least capacity elements without reallocating. If capacity is 0, the hash map will not allocate.

HashDoS resistance

The hash_builder normally use a fixed key by default and that does not allow the HashMap to be protected against attacks such as HashDoS. Users who require HashDoS resistance should explicitly use std::collections::hash_map::RandomState as the hasher when creating a HashMap.

The hash_builder passed should implement the BuildHasher trait for the HashMap to be useful, see its documentation for details.

Examples

use hashbrown::HashMap;
use hashbrown::DefaultHashBuilder;

let s = DefaultHashBuilder::default();
let mut map = HashMap::with_capacity_and_hasher(10, s);
assert_eq!(map.len(), 0);
assert!(map.capacity() >= 10);

map.insert(1, 2);

impl<K, V, S, A> HashMap<K, V, S, A>

fn reserve(self: &mut Self, additional: usize)

Reserves capacity for at least additional more elements to be inserted in the HashMap. The collection may reserve more space to avoid frequent reallocations.

Panics

Panics if the new capacity exceeds isize::MAX bytes and abort the program in case of allocation error. Use try_reserve instead if you want to handle memory allocation failure.

Examples

use hashbrown::HashMap;
let mut map: HashMap<&str, i32> = HashMap::new();
// Map is empty and doesn't allocate memory
assert_eq!(map.capacity(), 0);

map.reserve(10);

// And now map can hold at least 10 elements
assert!(map.capacity() >= 10);
fn try_reserve(self: &mut Self, additional: usize) -> Result<(), TryReserveError>

Tries to reserve capacity for at least additional more elements to be inserted in the given HashMap<K,V>. The collection may reserve more space to avoid frequent reallocations.

Errors

If the capacity overflows, or the allocator reports a failure, then an error is returned.

Examples

use hashbrown::HashMap;

let mut map: HashMap<&str, isize> = HashMap::new();
// Map is empty and doesn't allocate memory
assert_eq!(map.capacity(), 0);

map.try_reserve(10).expect("why is the test harness OOMing on 10 bytes?");

// And now map can hold at least 10 elements
assert!(map.capacity() >= 10);

If the capacity overflows, or the allocator reports a failure, then an error is returned:

# fn test() {
use hashbrown::HashMap;
use hashbrown::TryReserveError;
let mut map: HashMap<i32, i32> = HashMap::new();

match map.try_reserve(usize::MAX) {
    Err(error) => match error {
        TryReserveError::CapacityOverflow => {}
        _ => panic!("TryReserveError::AllocError ?"),
    },
    _ => panic!(),
}
# }
# fn main() {
#     #[cfg(not(miri))]
#     test()
# }
fn shrink_to_fit(self: &mut Self)

Shrinks the capacity of the map as much as possible. It will drop down as much as possible while maintaining the internal rules and possibly leaving some space in accordance with the resize policy.

Examples

use hashbrown::HashMap;

let mut map: HashMap<i32, i32> = HashMap::with_capacity(100);
map.insert(1, 2);
map.insert(3, 4);
assert!(map.capacity() >= 100);
map.shrink_to_fit();
assert!(map.capacity() >= 2);
fn shrink_to(self: &mut Self, min_capacity: usize)

Shrinks the capacity of the map with a lower limit. It will drop down no lower than the supplied limit while maintaining the internal rules and possibly leaving some space in accordance with the resize policy.

This function does nothing if the current capacity is smaller than the supplied minimum capacity.

Examples

use hashbrown::HashMap;

let mut map: HashMap<i32, i32> = HashMap::with_capacity(100);
map.insert(1, 2);
map.insert(3, 4);
assert!(map.capacity() >= 100);
map.shrink_to(10);
assert!(map.capacity() >= 10);
map.shrink_to(0);
assert!(map.capacity() >= 2);
map.shrink_to(10);
assert!(map.capacity() >= 2);
fn entry(self: &mut Self, key: K) -> Entry<'_, K, V, S, A>

Gets the given key's corresponding entry in the map for in-place manipulation.

Examples

use hashbrown::HashMap;

let mut letters = HashMap::new();

for ch in "a short treatise on fungi".chars() {
    let counter = letters.entry(ch).or_insert(0);
    *counter += 1;
}

assert_eq!(letters[&'s'], 2);
assert_eq!(letters[&'t'], 3);
assert_eq!(letters[&'u'], 1);
assert_eq!(letters.get(&'y'), None);
fn entry_ref<'a, 'b, Q>(self: &'a mut Self, key: &'b Q) -> EntryRef<'a, 'b, K, Q, V, S, A>
where
    Q: Hash + Equivalent<K> + ?Sized

Gets the given key's corresponding entry by reference in the map for in-place manipulation.

Examples

use hashbrown::HashMap;

let mut words: HashMap<String, usize> = HashMap::new();
let source = ["poneyland", "horseyland", "poneyland", "poneyland"];
for (i, &s) in source.iter().enumerate() {
    let counter = words.entry_ref(s).or_insert(0);
    *counter += 1;
}

assert_eq!(words["poneyland"], 3);
assert_eq!(words["horseyland"], 1);
fn get<Q>(self: &Self, k: &Q) -> Option<&V>
where
    Q: Hash + Equivalent<K> + ?Sized

Returns a reference to the value corresponding to the key.

The key may be any borrowed form of the map's key type, but Hash and Eq on the borrowed form must match those for the key type.

Examples

use hashbrown::HashMap;

let mut map = HashMap::new();
map.insert(1, "a");
assert_eq!(map.get(&1), Some(&"a"));
assert_eq!(map.get(&2), None);
fn get_key_value<Q>(self: &Self, k: &Q) -> Option<(&K, &V)>
where
    Q: Hash + Equivalent<K> + ?Sized

Returns the key-value pair corresponding to the supplied key.

The supplied key may be any borrowed form of the map's key type, but Hash and Eq on the borrowed form must match those for the key type.

Examples

use hashbrown::HashMap;

let mut map = HashMap::new();
map.insert(1, "a");
assert_eq!(map.get_key_value(&1), Some((&1, &"a")));
assert_eq!(map.get_key_value(&2), None);
fn get_key_value_mut<Q>(self: &mut Self, k: &Q) -> Option<(&K, &mut V)>
where
    Q: Hash + Equivalent<K> + ?Sized

Returns the key-value pair corresponding to the supplied key, with a mutable reference to value.

The supplied key may be any borrowed form of the map's key type, but Hash and Eq on the borrowed form must match those for the key type.

Examples

use hashbrown::HashMap;

let mut map = HashMap::new();
map.insert(1, "a");
let (k, v) = map.get_key_value_mut(&1).unwrap();
assert_eq!(k, &1);
assert_eq!(v, &mut "a");
*v = "b";
assert_eq!(map.get_key_value_mut(&1), Some((&1, &mut "b")));
assert_eq!(map.get_key_value_mut(&2), None);
fn contains_key<Q>(self: &Self, k: &Q) -> bool
where
    Q: Hash + Equivalent<K> + ?Sized

Returns true if the map contains a value for the specified key.

The key may be any borrowed form of the map's key type, but Hash and Eq on the borrowed form must match those for the key type.

Examples

use hashbrown::HashMap;

let mut map = HashMap::new();
map.insert(1, "a");
assert_eq!(map.contains_key(&1), true);
assert_eq!(map.contains_key(&2), false);
fn get_mut<Q>(self: &mut Self, k: &Q) -> Option<&mut V>
where
    Q: Hash + Equivalent<K> + ?Sized

Returns a mutable reference to the value corresponding to the key.

The key may be any borrowed form of the map's key type, but Hash and Eq on the borrowed form must match those for the key type.

Examples

use hashbrown::HashMap;

let mut map = HashMap::new();
map.insert(1, "a");
if let Some(x) = map.get_mut(&1) {
    *x = "b";
}
assert_eq!(map[&1], "b");

assert_eq!(map.get_mut(&2), None);
fn get_many_mut<Q, N: usize>(self: &mut Self, ks: [&Q; N]) -> [Option<&mut V>; N]
where
    Q: Hash + Equivalent<K> + ?Sized

Attempts to get mutable references to N values in the map at once.

Returns an array of length N with the results of each query. For soundness, at most one mutable reference will be returned to any value. None will be used if the key is missing.

Panics

Panics if any keys are overlapping.

Examples

use hashbrown::HashMap;

let mut libraries = HashMap::new();
libraries.insert("Bodleian Library".to_string(), 1602);
libraries.insert("Athenæum".to_string(), 1807);
libraries.insert("Herzogin-Anna-Amalia-Bibliothek".to_string(), 1691);
libraries.insert("Library of Congress".to_string(), 1800);

// Get Athenæum and Bodleian Library
let [Some(a), Some(b)] = libraries.get_many_mut([
    "Athenæum",
    "Bodleian Library",
]) else { panic!() };

// Assert values of Athenæum and Library of Congress
let got = libraries.get_many_mut([
    "Athenæum",
    "Library of Congress",
]);
assert_eq!(
    got,
    [
        Some(&mut 1807),
        Some(&mut 1800),
    ],
);

// Missing keys result in None
let got = libraries.get_many_mut([
    "Athenæum",
    "New York Public Library",
]);
assert_eq!(
    got,
    [
        Some(&mut 1807),
        None
    ]
);
use hashbrown::HashMap;

let mut libraries = HashMap::new();
libraries.insert("Athenæum".to_string(), 1807);

// Duplicate keys panic!
let got = libraries.get_many_mut([
    "Athenæum",
    "Athenæum",
]);
unsafe fn get_many_unchecked_mut<Q, N: usize>(self: &mut Self, ks: [&Q; N]) -> [Option<&mut V>; N]
where
    Q: Hash + Equivalent<K> + ?Sized

Attempts to get mutable references to N values in the map at once, without validating that the values are unique.

Returns an array of length N with the results of each query. None will be used if the key is missing.

For a safe alternative see get_many_mut.

Safety

Calling this method with overlapping keys is undefined behavior even if the resulting references are not used.

Examples

use hashbrown::HashMap;

let mut libraries = HashMap::new();
libraries.insert("Bodleian Library".to_string(), 1602);
libraries.insert("Athenæum".to_string(), 1807);
libraries.insert("Herzogin-Anna-Amalia-Bibliothek".to_string(), 1691);
libraries.insert("Library of Congress".to_string(), 1800);

// SAFETY: The keys do not overlap.
let [Some(a), Some(b)] = (unsafe { libraries.get_many_unchecked_mut([
    "Athenæum",
    "Bodleian Library",
]) }) else { panic!() };

// SAFETY: The keys do not overlap.
let got = unsafe { libraries.get_many_unchecked_mut([
    "Athenæum",
    "Library of Congress",
]) };
assert_eq!(
    got,
    [
        Some(&mut 1807),
        Some(&mut 1800),
    ],
);

// SAFETY: The keys do not overlap.
let got = unsafe { libraries.get_many_unchecked_mut([
    "Athenæum",
    "New York Public Library",
]) };
// Missing keys result in None
assert_eq!(got, [Some(&mut 1807), None]);
fn get_many_key_value_mut<Q, N: usize>(self: &mut Self, ks: [&Q; N]) -> [Option<(&K, &mut V)>; N]
where
    Q: Hash + Equivalent<K> + ?Sized

Attempts to get mutable references to N values in the map at once, with immutable references to the corresponding keys.

Returns an array of length N with the results of each query. For soundness, at most one mutable reference will be returned to any value. None will be used if the key is missing.

Panics

Panics if any keys are overlapping.

Examples

use hashbrown::HashMap;

let mut libraries = HashMap::new();
libraries.insert("Bodleian Library".to_string(), 1602);
libraries.insert("Athenæum".to_string(), 1807);
libraries.insert("Herzogin-Anna-Amalia-Bibliothek".to_string(), 1691);
libraries.insert("Library of Congress".to_string(), 1800);

let got = libraries.get_many_key_value_mut([
    "Bodleian Library",
    "Herzogin-Anna-Amalia-Bibliothek",
]);
assert_eq!(
    got,
    [
        Some((&"Bodleian Library".to_string(), &mut 1602)),
        Some((&"Herzogin-Anna-Amalia-Bibliothek".to_string(), &mut 1691)),
    ],
);
// Missing keys result in None
let got = libraries.get_many_key_value_mut([
    "Bodleian Library",
    "Gewandhaus",
]);
assert_eq!(got, [Some((&"Bodleian Library".to_string(), &mut 1602)), None]);
use hashbrown::HashMap;

let mut libraries = HashMap::new();
libraries.insert("Bodleian Library".to_string(), 1602);
libraries.insert("Herzogin-Anna-Amalia-Bibliothek".to_string(), 1691);

// Duplicate keys result in panic!
let got = libraries.get_many_key_value_mut([
    "Bodleian Library",
    "Herzogin-Anna-Amalia-Bibliothek",
    "Herzogin-Anna-Amalia-Bibliothek",
]);
unsafe fn get_many_key_value_unchecked_mut<Q, N: usize>(self: &mut Self, ks: [&Q; N]) -> [Option<(&K, &mut V)>; N]
where
    Q: Hash + Equivalent<K> + ?Sized

Attempts to get mutable references to N values in the map at once, with immutable references to the corresponding keys, without validating that the values are unique.

Returns an array of length N with the results of each query. None will be returned if any of the keys are missing.

For a safe alternative see get_many_key_value_mut.

Safety

Calling this method with overlapping keys is undefined behavior even if the resulting references are not used.

Examples

use hashbrown::HashMap;

let mut libraries = HashMap::new();
libraries.insert("Bodleian Library".to_string(), 1602);
libraries.insert("Athenæum".to_string(), 1807);
libraries.insert("Herzogin-Anna-Amalia-Bibliothek".to_string(), 1691);
libraries.insert("Library of Congress".to_string(), 1800);

let got = libraries.get_many_key_value_mut([
    "Bodleian Library",
    "Herzogin-Anna-Amalia-Bibliothek",
]);
assert_eq!(
    got,
    [
        Some((&"Bodleian Library".to_string(), &mut 1602)),
        Some((&"Herzogin-Anna-Amalia-Bibliothek".to_string(), &mut 1691)),
    ],
);
// Missing keys result in None
let got = libraries.get_many_key_value_mut([
    "Bodleian Library",
    "Gewandhaus",
]);
assert_eq!(
    got,
    [
        Some((&"Bodleian Library".to_string(), &mut 1602)),
        None,
    ],
);
fn insert(self: &mut Self, k: K, v: V) -> Option<V>

Inserts a key-value pair into the map.

If the map did not have this key present, None is returned.

If the map did have this key present, the value is updated, and the old value is returned. The key is not updated, though; this matters for types that can be == without being identical. See the std::collections module-level documentation for more.

Examples

use hashbrown::HashMap;

let mut map = HashMap::new();
assert_eq!(map.insert(37, "a"), None);
assert_eq!(map.is_empty(), false);

map.insert(37, "b");
assert_eq!(map.insert(37, "c"), Some("b"));
assert_eq!(map[&37], "c");
unsafe fn insert_unique_unchecked(self: &mut Self, k: K, v: V) -> (&K, &mut V)

Insert a key-value pair into the map without checking if the key already exists in the map.

This operation is faster than regular insert, because it does not perform lookup before insertion.

This operation is useful during initial population of the map. For example, when constructing a map from another map, we know that keys are unique.

Returns a reference to the key and value just inserted.

Safety

This operation is safe if a key does not exist in the map.

However, if a key exists in the map already, the behavior is unspecified: this operation may panic, loop forever, or any following operation with the map may panic, loop forever or return arbitrary result.

That said, this operation (and following operations) are guaranteed to not violate memory safety.

However this operation is still unsafe because the resulting HashMap may be passed to unsafe code which does expect the map to behave correctly, and would cause unsoundness as a result.

Examples

use hashbrown::HashMap;

let mut map1 = HashMap::new();
assert_eq!(map1.insert(1, "a"), None);
assert_eq!(map1.insert(2, "b"), None);
assert_eq!(map1.insert(3, "c"), None);
assert_eq!(map1.len(), 3);

let mut map2 = HashMap::new();

for (key, value) in map1.into_iter() {
    unsafe {
        map2.insert_unique_unchecked(key, value);
    }
}

let (key, value) = unsafe { map2.insert_unique_unchecked(4, "d") };
assert_eq!(key, &4);
assert_eq!(value, &mut "d");
*value = "e";

assert_eq!(map2[&1], "a");
assert_eq!(map2[&2], "b");
assert_eq!(map2[&3], "c");
assert_eq!(map2[&4], "e");
assert_eq!(map2.len(), 4);
fn try_insert(self: &mut Self, key: K, value: V) -> Result<&mut V, OccupiedError<'_, K, V, S, A>>

Tries to insert a key-value pair into the map, and returns a mutable reference to the value in the entry.

Errors

If the map already had this key present, nothing is updated, and an error containing the occupied entry and the value is returned.

Examples

Basic usage:

use hashbrown::HashMap;
use hashbrown::hash_map::OccupiedError;

let mut map = HashMap::new();
assert_eq!(map.try_insert(37, "a").unwrap(), &"a");

match map.try_insert(37, "b") {
    Err(OccupiedError { entry, value }) => {
        assert_eq!(entry.key(), &37);
        assert_eq!(entry.get(), &"a");
        assert_eq!(value, "b");
    }
    _ => panic!()
}
fn remove<Q>(self: &mut Self, k: &Q) -> Option<V>
where
    Q: Hash + Equivalent<K> + ?Sized

Removes a key from the map, returning the value at the key if the key was previously in the map. Keeps the allocated memory for reuse.

The key may be any borrowed form of the map's key type, but Hash and Eq on the borrowed form must match those for the key type.

Examples

use hashbrown::HashMap;

let mut map = HashMap::new();
// The map is empty
assert!(map.is_empty() && map.capacity() == 0);

map.insert(1, "a");

assert_eq!(map.remove(&1), Some("a"));
assert_eq!(map.remove(&1), None);

// Now map holds none elements
assert!(map.is_empty());
fn remove_entry<Q>(self: &mut Self, k: &Q) -> Option<(K, V)>
where
    Q: Hash + Equivalent<K> + ?Sized

Removes a key from the map, returning the stored key and value if the key was previously in the map. Keeps the allocated memory for reuse.

The key may be any borrowed form of the map's key type, but Hash and Eq on the borrowed form must match those for the key type.

Examples

use hashbrown::HashMap;

let mut map = HashMap::new();
// The map is empty
assert!(map.is_empty() && map.capacity() == 0);

map.insert(1, "a");

assert_eq!(map.remove_entry(&1), Some((1, "a")));
assert_eq!(map.remove(&1), None);

// Now map hold none elements
assert!(map.is_empty());
fn allocation_size(self: &Self) -> usize

Returns the total amount of memory allocated internally by the hash set, in bytes.

The returned number is informational only. It is intended to be primarily used for memory profiling.

impl<K, V, S, A: Allocator> HashMap<K, V, S, A>

fn allocator(self: &Self) -> &A

Returns a reference to the underlying allocator.

const fn with_hasher_in(hash_builder: S, alloc: A) -> Self

Creates an empty HashMap which will use the given hash builder to hash keys. It will be allocated with the given allocator.

The hash map is initially created with a capacity of 0, so it will not allocate until it is first inserted into.

HashDoS resistance

The hash_builder normally use a fixed key by default and that does not allow the HashMap to be protected against attacks such as HashDoS. Users who require HashDoS resistance should explicitly use std::collections::hash_map::RandomState as the hasher when creating a HashMap.

Examples

use hashbrown::HashMap;
use hashbrown::DefaultHashBuilder;

let s = DefaultHashBuilder::default();
let mut map = HashMap::with_hasher(s);
map.insert(1, 2);
fn with_capacity_and_hasher_in(capacity: usize, hash_builder: S, alloc: A) -> Self

Creates an empty HashMap with the specified capacity, using hash_builder to hash the keys. It will be allocated with the given allocator.

The hash map will be able to hold at least capacity elements without reallocating. If capacity is 0, the hash map will not allocate.

HashDoS resistance

The hash_builder normally use a fixed key by default and that does not allow the HashMap to be protected against attacks such as HashDoS. Users who require HashDoS resistance should explicitly use std::collections::hash_map::RandomState as the hasher when creating a HashMap.

Examples

use hashbrown::HashMap;
use hashbrown::DefaultHashBuilder;

let s = DefaultHashBuilder::default();
let mut map = HashMap::with_capacity_and_hasher(10, s);
map.insert(1, 2);
fn hasher(self: &Self) -> &S

Returns a reference to the map's BuildHasher.

Examples

use hashbrown::HashMap;
use hashbrown::DefaultHashBuilder;

let hasher = DefaultHashBuilder::default();
let map: HashMap<i32, i32> = HashMap::with_hasher(hasher);
let hasher: &DefaultHashBuilder = map.hasher();
fn capacity(self: &Self) -> usize

Returns the number of elements the map can hold without reallocating.

This number is a lower bound; the HashMap<K, V> might be able to hold more, but is guaranteed to be able to hold at least this many.

Examples

use hashbrown::HashMap;
let map: HashMap<i32, i32> = HashMap::with_capacity(100);
assert_eq!(map.len(), 0);
assert!(map.capacity() >= 100);
fn keys(self: &Self) -> Keys<'_, K, V>

An iterator visiting all keys in arbitrary order. The iterator element type is &'a K.

Examples

use hashbrown::HashMap;

let mut map = HashMap::new();
map.insert("a", 1);
map.insert("b", 2);
map.insert("c", 3);
assert_eq!(map.len(), 3);
let mut vec: Vec<&str> = Vec::new();

for key in map.keys() {
    println!("{}", key);
    vec.push(*key);
}

// The `Keys` iterator produces keys in arbitrary order, so the
// keys must be sorted to test them against a sorted array.
vec.sort_unstable();
assert_eq!(vec, ["a", "b", "c"]);

assert_eq!(map.len(), 3);
fn values(self: &Self) -> Values<'_, K, V>

An iterator visiting all values in arbitrary order. The iterator element type is &'a V.

Examples

use hashbrown::HashMap;

let mut map = HashMap::new();
map.insert("a", 1);
map.insert("b", 2);
map.insert("c", 3);
assert_eq!(map.len(), 3);
let mut vec: Vec<i32> = Vec::new();

for val in map.values() {
    println!("{}", val);
    vec.push(*val);
}

// The `Values` iterator produces values in arbitrary order, so the
// values must be sorted to test them against a sorted array.
vec.sort_unstable();
assert_eq!(vec, [1, 2, 3]);

assert_eq!(map.len(), 3);
fn values_mut(self: &mut Self) -> ValuesMut<'_, K, V>

An iterator visiting all values mutably in arbitrary order. The iterator element type is &'a mut V.

Examples

use hashbrown::HashMap;

let mut map = HashMap::new();

map.insert("a", 1);
map.insert("b", 2);
map.insert("c", 3);

for val in map.values_mut() {
    *val = *val + 10;
}

assert_eq!(map.len(), 3);
let mut vec: Vec<i32> = Vec::new();

for val in map.values() {
    println!("{}", val);
    vec.push(*val);
}

// The `Values` iterator produces values in arbitrary order, so the
// values must be sorted to test them against a sorted array.
vec.sort_unstable();
assert_eq!(vec, [11, 12, 13]);

assert_eq!(map.len(), 3);
fn iter(self: &Self) -> Iter<'_, K, V>

An iterator visiting all key-value pairs in arbitrary order. The iterator element type is (&'a K, &'a V).

Examples

use hashbrown::HashMap;

let mut map = HashMap::new();
map.insert("a", 1);
map.insert("b", 2);
map.insert("c", 3);
assert_eq!(map.len(), 3);
let mut vec: Vec<(&str, i32)> = Vec::new();

for (key, val) in map.iter() {
    println!("key: {} val: {}", key, val);
    vec.push((*key, *val));
}

// The `Iter` iterator produces items in arbitrary order, so the
// items must be sorted to test them against a sorted array.
vec.sort_unstable();
assert_eq!(vec, [("a", 1), ("b", 2), ("c", 3)]);

assert_eq!(map.len(), 3);
fn iter_mut(self: &mut Self) -> IterMut<'_, K, V>

An iterator visiting all key-value pairs in arbitrary order, with mutable references to the values. The iterator element type is (&'a K, &'a mut V).

Examples

use hashbrown::HashMap;

let mut map = HashMap::new();
map.insert("a", 1);
map.insert("b", 2);
map.insert("c", 3);

// Update all values
for (_, val) in map.iter_mut() {
    *val *= 2;
}

assert_eq!(map.len(), 3);
let mut vec: Vec<(&str, i32)> = Vec::new();

for (key, val) in &map {
    println!("key: {} val: {}", key, val);
    vec.push((*key, *val));
}

// The `Iter` iterator produces items in arbitrary order, so the
// items must be sorted to test them against a sorted array.
vec.sort_unstable();
assert_eq!(vec, [("a", 2), ("b", 4), ("c", 6)]);

assert_eq!(map.len(), 3);
fn len(self: &Self) -> usize

Returns the number of elements in the map.

Examples

use hashbrown::HashMap;

let mut a = HashMap::new();
assert_eq!(a.len(), 0);
a.insert(1, "a");
assert_eq!(a.len(), 1);
fn is_empty(self: &Self) -> bool

Returns true if the map contains no elements.

Examples

use hashbrown::HashMap;

let mut a = HashMap::new();
assert!(a.is_empty());
a.insert(1, "a");
assert!(!a.is_empty());
fn drain(self: &mut Self) -> Drain<'_, K, V, A>

Clears the map, returning all key-value pairs as an iterator. Keeps the allocated memory for reuse.

If the returned iterator is dropped before being fully consumed, it drops the remaining key-value pairs. The returned iterator keeps a mutable borrow on the vector to optimize its implementation.

Examples

use hashbrown::HashMap;

let mut a = HashMap::new();
a.insert(1, "a");
a.insert(2, "b");
let capacity_before_drain = a.capacity();

for (k, v) in a.drain().take(1) {
    assert!(k == 1 || k == 2);
    assert!(v == "a" || v == "b");
}

// As we can see, the map is empty and contains no element.
assert!(a.is_empty() && a.len() == 0);
// But map capacity is equal to old one.
assert_eq!(a.capacity(), capacity_before_drain);

let mut a = HashMap::new();
a.insert(1, "a");
a.insert(2, "b");

{   // Iterator is dropped without being consumed.
    let d = a.drain();
}

// But the map is empty even if we do not use Drain iterator.
assert!(a.is_empty());
fn retain<F>(self: &mut Self, f: F)
where
    F: FnMut(&K, &mut V) -> bool

Retains only the elements specified by the predicate. Keeps the allocated memory for reuse.

In other words, remove all pairs (k, v) such that f(&k, &mut v) returns false. The elements are visited in unsorted (and unspecified) order.

Examples

use hashbrown::HashMap;

let mut map: HashMap<i32, i32> = (0..8).map(|x|(x, x*10)).collect();
assert_eq!(map.len(), 8);

map.retain(|&k, _| k % 2 == 0);

// We can see, that the number of elements inside map is changed.
assert_eq!(map.len(), 4);

let mut vec: Vec<(i32, i32)> = map.iter().map(|(&k, &v)| (k, v)).collect();
vec.sort_unstable();
assert_eq!(vec, [(0, 0), (2, 20), (4, 40), (6, 60)]);
fn extract_if<F>(self: &mut Self, f: F) -> ExtractIf<'_, K, V, F, A>
where
    F: FnMut(&K, &mut V) -> bool

Drains elements which are true under the given predicate, and returns an iterator over the removed items.

In other words, move all pairs (k, v) such that f(&k, &mut v) returns true out into another iterator.

Note that extract_if lets you mutate every value in the filter closure, regardless of whether you choose to keep or remove it.

If the returned ExtractIf is not exhausted, e.g. because it is dropped without iterating or the iteration short-circuits, then the remaining elements will be retained. Use retain() with a negated predicate if you do not need the returned iterator.

Keeps the allocated memory for reuse.

Examples

use hashbrown::HashMap;

let mut map: HashMap<i32, i32> = (0..8).map(|x| (x, x)).collect();

let drained: HashMap<i32, i32> = map.extract_if(|k, _v| k % 2 == 0).collect();

let mut evens = drained.keys().cloned().collect::<Vec<_>>();
let mut odds = map.keys().cloned().collect::<Vec<_>>();
evens.sort();
odds.sort();

assert_eq!(evens, vec![0, 2, 4, 6]);
assert_eq!(odds, vec![1, 3, 5, 7]);

let mut map: HashMap<i32, i32> = (0..8).map(|x| (x, x)).collect();

{   // Iterator is dropped without being consumed.
    let d = map.extract_if(|k, _v| k % 2 != 0);
}

// ExtractIf was not exhausted, therefore no elements were drained.
assert_eq!(map.len(), 8);
fn clear(self: &mut Self)

Clears the map, removing all key-value pairs. Keeps the allocated memory for reuse.

Examples

use hashbrown::HashMap;

let mut a = HashMap::new();
a.insert(1, "a");
let capacity_before_clear = a.capacity();

a.clear();

// Map is empty.
assert!(a.is_empty());
// But map capacity is equal to old one.
assert_eq!(a.capacity(), capacity_before_clear);
fn into_keys(self: Self) -> IntoKeys<K, V, A>

Creates a consuming iterator visiting all the keys in arbitrary order. The map cannot be used after calling this. The iterator element type is K.

Examples

use hashbrown::HashMap;

let mut map = HashMap::new();
map.insert("a", 1);
map.insert("b", 2);
map.insert("c", 3);

let mut vec: Vec<&str> = map.into_keys().collect();

// The `IntoKeys` iterator produces keys in arbitrary order, so the
// keys must be sorted to test them against a sorted array.
vec.sort_unstable();
assert_eq!(vec, ["a", "b", "c"]);
fn into_values(self: Self) -> IntoValues<K, V, A>

Creates a consuming iterator visiting all the values in arbitrary order. The map cannot be used after calling this. The iterator element type is V.

Examples

use hashbrown::HashMap;

let mut map = HashMap::new();
map.insert("a", 1);
map.insert("b", 2);
map.insert("c", 3);

let mut vec: Vec<i32> = map.into_values().collect();

// The `IntoValues` iterator produces values in arbitrary order, so
// the values must be sorted to test them against a sorted array.
vec.sort_unstable();
assert_eq!(vec, [1, 2, 3]);

impl<'a, K, V, S, A> Extend for HashMap<K, V, S, A>

fn extend<T: IntoIterator<Item = (&'a K, &'a V)>>(self: &mut Self, iter: T)

Inserts all new key-values from the iterator to existing HashMap<K, V, S, A>. Replace values with existing keys with new values returned from the iterator. The keys and values must implement Copy trait.

Examples

use hashbrown::hash_map::HashMap;

let mut map = HashMap::new();
map.insert(1, 100);

let arr = [(1, 1), (2, 2)];
let some_iter = arr.iter().map(|(k, v)| (k, v));
map.extend(some_iter);
// Replace values with existing keys with new values returned from the iterator.
// So that the map.get(&1) doesn't return Some(&100).
assert_eq!(map.get(&1), Some(&1));

let some_vec: Vec<_> = vec![(3, 3), (4, 4)];
map.extend(some_vec.iter().map(|(k, v)| (k, v)));

let some_arr = [(5, 5), (6, 6)];
map.extend(some_arr.iter().map(|(k, v)| (k, v)));

// You can also extend from another HashMap
let mut new_map = HashMap::new();
new_map.extend(&map);
assert_eq!(new_map, map);

let mut vec: Vec<_> = new_map.into_iter().collect();
// The `IntoIter` iterator produces items in arbitrary order, so the
// items must be sorted to test them against a sorted array.
vec.sort_unstable();
assert_eq!(vec, [(1, 1), (2, 2), (3, 3), (4, 4), (5, 5), (6, 6)]);

impl<'a, K, V, S, A> Extend for HashMap<K, V, S, A>

fn extend<T: IntoIterator<Item = &'a (K, V)>>(self: &mut Self, iter: T)

Inserts all new key-values from the iterator to existing HashMap<K, V, S, A>. Replace values with existing keys with new values returned from the iterator. The keys and values must implement Copy trait.

Examples

use hashbrown::hash_map::HashMap;

let mut map = HashMap::new();
map.insert(1, 100);

let arr = [(1, 1), (2, 2)];
let some_iter = arr.iter();
map.extend(some_iter);
// Replace values with existing keys with new values returned from the iterator.
// So that the map.get(&1) doesn't return Some(&100).
assert_eq!(map.get(&1), Some(&1));

let some_vec: Vec<_> = vec![(3, 3), (4, 4)];
map.extend(&some_vec);

let some_arr = [(5, 5), (6, 6)];
map.extend(&some_arr);

let mut vec: Vec<_> = map.into_iter().collect();
// The `IntoIter` iterator produces items in arbitrary order, so the
// items must be sorted to test them against a sorted array.
vec.sort_unstable();
assert_eq!(vec, [(1, 1), (2, 2), (3, 3), (4, 4), (5, 5), (6, 6)]);

impl<K, Q, V, S, A> Index for HashMap<K, V, S, A>

fn index(self: &Self, key: &Q) -> &V

Returns a reference to the value corresponding to the supplied key.

Panics

Panics if the key is not present in the HashMap.

Examples

use hashbrown::HashMap;

let map: HashMap<_, _> = [("a", "One"), ("b", "Two")].into();

assert_eq!(map[&"a"], "One");
assert_eq!(map[&"b"], "Two");

impl<K, V, S, A> Debug for HashMap<K, V, S, A>

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl<K, V, S, A> Default for HashMap<K, V, S, A>

fn default() -> Self

Creates an empty HashMap<K, V, S, A>, with the Default value for the hasher and allocator.

Examples

use hashbrown::HashMap;
use std::collections::hash_map::RandomState;

// You can specify all types of HashMap, including hasher and allocator.
// Created map is empty and don't allocate memory
let map: HashMap<u32, String> = Default::default();
assert_eq!(map.capacity(), 0);
let map: HashMap<u32, String, RandomState> = HashMap::default();
assert_eq!(map.capacity(), 0);

impl<K, V, S, A> Eq for HashMap<K, V, S, A>

impl<K, V, S, A> Extend for HashMap<K, V, S, A>

fn extend<T: IntoIterator<Item = (K, V)>>(self: &mut Self, iter: T)

Inserts all new key-values from the iterator to existing HashMap<K, V, S, A>. Replace values with existing keys with new values returned from the iterator.

Examples

use hashbrown::hash_map::HashMap;

let mut map = HashMap::new();
map.insert(1, 100);

let some_iter = [(1, 1), (2, 2)].into_iter();
map.extend(some_iter);
// Replace values with existing keys with new values returned from the iterator.
// So that the map.get(&1) doesn't return Some(&100).
assert_eq!(map.get(&1), Some(&1));

let some_vec: Vec<_> = vec![(3, 3), (4, 4)];
map.extend(some_vec);

let some_arr = [(5, 5), (6, 6)];
map.extend(some_arr);
let old_map_len = map.len();

// You can also extend from another HashMap
let mut new_map = HashMap::new();
new_map.extend(map);
assert_eq!(new_map.len(), old_map_len);

let mut vec: Vec<_> = new_map.into_iter().collect();
// The `IntoIter` iterator produces items in arbitrary order, so the
// items must be sorted to test them against a sorted array.
vec.sort_unstable();
assert_eq!(vec, [(1, 1), (2, 2), (3, 3), (4, 4), (5, 5), (6, 6)]);

impl<K, V, S, A> Freeze for HashMap<K, V, S, A>

impl<K, V, S, A> FromIterator for HashMap<K, V, S, A>

fn from_iter<T: IntoIterator<Item = (K, V)>>(iter: T) -> Self

impl<K, V, S, A> PartialEq for HashMap<K, V, S, A>

fn eq(self: &Self, other: &Self) -> bool

impl<K, V, S, A> RefUnwindSafe for HashMap<K, V, S, A>

impl<K, V, S, A> Send for HashMap<K, V, S, A>

impl<K, V, S, A> Sync for HashMap<K, V, S, A>

impl<K, V, S, A> Unpin for HashMap<K, V, S, A>

impl<K, V, S, A> UnsafeUnpin for HashMap<K, V, S, A>

impl<K, V, S, A> UnwindSafe for HashMap<K, V, S, A>

impl<K, V, S, A: Allocator> IntoIterator for HashMap<K, V, S, A>

fn into_iter(self: Self) -> IntoIter<K, V, A>

Creates a consuming iterator, that is, one that moves each key-value pair out of the map in arbitrary order. The map cannot be used after calling this.

Examples

use hashbrown::HashMap;

let map: HashMap<_, _> = [("a", 1), ("b", 2), ("c", 3)].into();

// Not possible with .iter()
let mut vec: Vec<(&str, i32)> = map.into_iter().collect();
// The `IntoIter` iterator produces items in arbitrary order, so
// the items must be sorted to test them against a sorted array.
vec.sort_unstable();
assert_eq!(vec, [("a", 1), ("b", 2), ("c", 3)]);

impl<K: Clone, V: Clone, S: Clone, A: Allocator + Clone> Clone for HashMap<K, V, S, A>

fn clone(self: &Self) -> Self
fn clone_from(self: &mut Self, source: &Self)

impl<Q, K> Equivalent for HashMap<K, V, S, A>

fn equivalent(self: &Self, key: &K) -> bool

impl<T> Any for HashMap<K, V, S, A>

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for HashMap<K, V, S, A>

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for HashMap<K, V, S, A>

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> CloneToUninit for HashMap<K, V, S, A>

unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)

impl<T> From for HashMap<K, V, S, A>

fn from(t: T) -> T

Returns the argument unchanged.

impl<T> ToOwned for HashMap<K, V, S, A>

fn to_owned(self: &Self) -> T
fn clone_into(self: &Self, target: &mut T)

impl<T, U> Into for HashMap<K, V, S, A>

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for HashMap<K, V, S, A>

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for HashMap<K, V, S, A>

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>